@@ -7,16 +7,21 @@
 using namespace std;
 using namespace Rcpp;
 
+static const double min_data_fraction = 0.1;
+
 // Implements R's type=8
-double c_quantile(double* data, const int n, const double quantile, bool sorted=false) {
-  if(n < 2 || quantile < 0 || quantile > 1) {
+double c_quantile(double* data, const int n, const double quantile, const bool sorted=false) {
+  if(n == 0 || quantile < 0 || quantile > 1) {
     return(R_NaReal);
   }
+
+  if(n == 1)
+    return(data[0]);
   
   // Constants for quantiles. Can be modified if needed.
   const double a = 1.0/3.0;
   const double b = 1.0/3.0;
-  double* data_end = data + n;
+  double* const data_end = data + n;
   
   const double fuzz = 4 * numeric_limits<double>::epsilon();
   const double nppm = a + quantile * (n + 1 - a - b) - 1;
@@ -79,7 +84,7 @@ public:
   const int* notna_map;
   const int half_win;
 
-  ClimdexBootstrapper(const double* dat, const int* notna_map, const int win_size, const int nyr, const int dpy): dat(dat), notna_map(notna_map), win_size(win_size), nyr(nyr), dpy(dpy), half_win(win_size / 2) { }
+  ClimdexBootstrapper(const double* dat, const int* notna_map, const int win_size, const int nyr, const int dpy): win_size(win_size), nyr(nyr), dpy(dpy), dat(dat), notna_map(notna_map), half_win(win_size / 2) { }
   
   // NOTE: Takes data with floor(win_size / 2) elements attached to beginning and end.
   // Extracts an n-day window into the data, removing NAs and generating a 2-tuple.
@@ -226,15 +231,15 @@ public:
       last_in_idx = idx;
     }
 
-    if(last_in_idx + 1 < in_data.size()) {
+    if((unsigned int)(last_in_idx + 1) < in_data.size()) {
       copy(&in_data[last_in_idx + 1], &in_data[in_data.size()], &out[next_out_idx]);
     }
   }
 
   vector<double> get_data_only(const vector<DatYrTuple>& in_dat) {
     vector<double> out_dat(in_dat.size());
-    const int dat_size = in_dat.size();
-    for(int i = 0; i < in_dat.size(); ++i)
+    const unsigned int dat_size = in_dat.size();
+    for(unsigned int i = 0; i < dat_size; ++i)
       out_dat[i] = in_dat[i].dat;
     return out_dat;
   }
@@ -253,9 +258,10 @@ RcppExport SEXP c_quantile2(SEXP data_, SEXP quantile_) {
   return res;
 }
 
-RcppExport SEXP running_quantile_windowed_bootstrap(SEXP data_, SEXP n_, SEXP q_, SEXP dpy_) {
+RcppExport SEXP running_quantile_windowed_bootstrap(SEXP data_, SEXP n_, SEXP q_, SEXP dpy_, SEXP min_fraction_) {
   const int win_size = as<int>(n_);
   const int days_per_year = as<int>(dpy_);
+  const double min_fraction = as<double>(min_fraction_);
   const NumericVector q(q_);
   const NumericVector data(data_);
   const int nq = q.size();
@@ -265,6 +271,7 @@ RcppExport SEXP running_quantile_windowed_bootstrap(SEXP data_, SEXP n_, SEXP q_
   const int win_border = win_size / 2;
   const int true_data_length = data_length - 2 * win_border;
   const int num_years = (int)ceil((double)true_data_length / (double)days_per_year);
+  const int slab_size = win_size * num_years;
   
   const int day_mul = 1;
   const int rmyr_mul = days_per_year;
@@ -277,7 +284,7 @@ RcppExport SEXP running_quantile_windowed_bootstrap(SEXP data_, SEXP n_, SEXP q_
   
   // Comment for preservation of sanity...
   // The input data does not stay on day 'day'; it starts on day 'day' - win_border. This gets around the problem of an off-by-two error.
-  vector<double> rep_dat(win_size * num_years);
+  vector<double> rep_dat(slab_size);
   for(int day = 0; day < days_per_year; ++day) {
     vector<DatYrTuple> win_tuples(bs.extract_window_with_year(day));
     std::sort(win_tuples.begin(), win_tuples.end());
@@ -292,9 +299,12 @@ RcppExport SEXP running_quantile_windowed_bootstrap(SEXP data_, SEXP n_, SEXP q_
 	if(dup_year != rm_year) {
 	  bs.replace_data_year(win_dat, rep_dat, yrs_index, rm_year, dup_year, day);
 	  const int off_day_rmyr_dupyr = off_day_rmyr + dup_idx * dupyr_mul;
-	  const int nq_day = nq * day;
-	  for(int q_idx = 0; q_idx < nq; ++q_idx)
-	    quantiles[off_day_rmyr_dupyr + q_idx * q_mul] = c_quantile(&rep_dat[0], rep_dat.size(), q[q_idx], true);
+	  if(((double)rep_dat.size() / (double)slab_size) < min_fraction)
+	    for(int q_idx = 0; q_idx < nq; ++q_idx)
+	      quantiles[off_day_rmyr_dupyr + q_idx * q_mul] = R_NaReal;
+	  else
+	    for(int q_idx = 0; q_idx < nq; ++q_idx)
+	      quantiles[off_day_rmyr_dupyr + q_idx * q_mul] = c_quantile(&rep_dat[0], rep_dat.size(), q[q_idx], true);
 	  ++dup_idx;
 	}
       }
@@ -306,9 +316,10 @@ RcppExport SEXP running_quantile_windowed_bootstrap(SEXP data_, SEXP n_, SEXP q_
 
 // Expects data in date sequence
 //void running_quantile_windowed_365day(const double* data, double* quantiles, const int* n, const double* q, const int* data_length, const int* num_quantiles) {
-RcppExport SEXP running_quantile_windowed(SEXP data_, SEXP n_, SEXP q_, SEXP dpy_) {
+RcppExport SEXP running_quantile_windowed(SEXP data_, SEXP n_, SEXP q_, SEXP dpy_, SEXP min_fraction_) {
   const int win_size = as<int>(n_);
   const int days_per_year = as<int>(dpy_);
+  const double min_fraction = as<double>(min_fraction_);
   const NumericVector q(q_);
   const NumericVector data(data_);
   const int nq = q.size();
@@ -318,11 +329,12 @@ RcppExport SEXP running_quantile_windowed(SEXP data_, SEXP n_, SEXP q_, SEXP dpy
   const int win_border = win_size / 2;
   const int true_data_length = data_length - 2 * win_border;
   const int num_years = (int)ceil((double)true_data_length / (double)days_per_year);
-  
+  const int slab_size = win_size * num_years;
+
   NumericVector quantiles(nq * days_per_year);
   
   // This data will be 2 dimensional, row major, with the major dimension being the day of the year
-  double* buf = new double[num_years * win_size];
+  double* buf = new double[slab_size];
   
   LogicalVector notna_map = !is_na(data);
   
@@ -342,8 +354,12 @@ RcppExport SEXP running_quantile_windowed(SEXP data_, SEXP n_, SEXP q_, SEXP dpy
     }
     // Quantiles on said buffer
     const int nq_day = nq * day;
-    for(int q_idx = 0; q_idx < nq; ++q_idx)
-      quantiles[nq_day + q_idx] = c_quantile(buf, count, q[q_idx]);
+    if(((double)count / (double)slab_size) < min_fraction)
+      for(int q_idx = 0; q_idx < nq; ++q_idx)
+	quantiles[nq_day + q_idx] = R_NaReal;
+    else
+      for(int q_idx = 0; q_idx < nq; ++q_idx)
+	quantiles[nq_day + q_idx] = c_quantile(buf, count, q[q_idx]);
   }
   delete[] buf;
   
